# -*- coding: utf-8 -*-
"""
Created on 2020/4/21 11:42 PM
---------
@summary: 基于内存的队列，代替redis
---------
@author: Boris
@email: boris_liu@foxmail.com
"""
from queue import PriorityQueue


class MemoryDB:
    def __init__(self):
        self.priority_queue = PriorityQueue()

    def add(self, item):
        """
        添加任务
        :param item: 数据: 支持小于号比较的类 或者 （priority, item）
        :return:
        """
        self.priority_queue.put(item)

    def get(self):
        """
        获取任务
        :return:
        """
        try:
            item = self.priority_queue.get_nowait()
            return item
        except:
            return

    def empty(self):
        return self.priority_queue.empty()



import signal
import time

def set_timeout(num):
    def wrap(func):
        def handle(
            signum, frame
        ):  # 收到信号 SIGALRM 后的回调函数，第一个参数是信号的数字，第二个参数是the interrupted stack frame.
            raise RuntimeError

        def to_do(*args, **kwargs):
            signal.signal(signal.SIGALRM, handle)  # 设置信号和回调函数
            signal.alarm(num)  # 设置 num 秒的闹钟
            print("start alarm signal.")
            r = func(*args, **kwargs)
            print("close alarm signal.")
            signal.alarm(0)  # 关闭闹钟
            return r

        return to_do

    return wrap

@set_timeout(1)
def test():
    print(1)
    time.sleep(2)
    print(2)

test()


# -*- coding: utf-8 -*-
"""
Created on 2020/5/9 12:37 AM
---------
@summary:
---------
@author: Boris
@email: boris_liu@foxmail.com
"""

import json
import re
import sys

import IPython

from feapder import Request


def request(**kwargs):
    kwargs.setdefault("proxies", None)
    response = Request(**kwargs).get_response()
    print(response)

    IPython.embed(header="now you can use response")


def fetch_url(url):
    request(url=url)


def fetch_curl(curl_args):
    """
    解析及抓取curl请求
    :param curl_args:
    [url, '-H', 'xxx', '-H', 'xxx', '--data-binary', '{"xxx":"xxx"}', '--compressed']
    :return:
    """
    url = curl_args[0]
    curl_args.pop(0)

    headers = {}
    data = {}
    for i in range(0, len(curl_args), 2):
        if curl_args[i] == "-H":
            regex = "([^:\s]*)[:|\s]*(.*)"
            result = re.search(regex, curl_args[i + 1], re.S).groups()
            if result[0] in headers:
                headers[result[0]] = headers[result[0]] + "&" + result[1]
            else:
                headers[result[0]] = result[1].strip()

        elif curl_args[i] == "--data-binary":
            data = json.loads(curl_args[i + 1])

    request(url=url, data=data, headers=headers)


def usage():
    """
下载调试器

usage: feapder shell [options] [args]

optional arguments:
  -u, --url     抓取指定url
  -c, --curl    抓取curl格式的请求

    """
    print(usage.__doc__)
    sys.exit()


def main():
    args = sys.argv
    if len(args) < 3:
        usage()

    elif args[1] in ("-h", "--help"):
        usage()

    elif args[1] in ("-u", "--url"):
        fetch_url(args[2])

    elif args[1] in ("-c", "--curl"):
        fetch_curl(args[2:])

    else:
        usage()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Created on 2018-07-26 22:28:10
---------
@summary: 定义实体
---------
@author: Boris
@email:  boris_liu@foxmail.com
"""

import re

import feapder.utils.tools as tools


class ItemMetaclass(type):
    def __new__(cls, name, bases, attrs):
        attrs.setdefault("__name__", None)
        attrs.setdefault("__table_name__", None)
        attrs.setdefault("__name_underline__", None)
        attrs.setdefault("__update_key__", None)
        attrs.setdefault("__unique_key__", None)

        return type.__new__(cls, name, bases, attrs)


class Item(metaclass=ItemMetaclass):
    __unique_key__ = []

    def __init__(self, **kwargs):
        self.__dict__ = kwargs

    def __repr__(self):
        return "<{}: {}>".format(self.item_name, tools.dumps_json(self.to_dict))

    def __getitem__(self, key):
        return self.__dict__[key]

    def __setitem__(self, key, value):
        self.__dict__[key] = value

    def pre_to_db(self):
        """
        入库前的处理
        """
        pass

    @property
    def to_dict(self):
        propertys = {}
        for key, value in self.__dict__.items():
            if key not in (
                "__name__",
                "__table_name__",
                "__name_underline__",
                "__update_key__",
                "__unique_key__",
            ):
                if key.startswith(f"_{self.__class__.__name__}"):
                    key = key.replace(f"_{self.__class__.__name__}", "")
                propertys[key] = value

        return propertys

    def to_sql(self, auto_update=False, update_columns=()):
        return tools.make_insert_sql(
            self.table_name, self.to_dict, auto_update, update_columns
        )

    @property
    def item_name(self):
        return self.__name__ or self.__class__.__name__

    @item_name.setter
    def item_name(self, name):
        self.__name__ = name
        self.__table_name__ = re.sub("_item$", "", self.name_underline)

    @property
    def table_name(self):
        if not self.__table_name__:
            self.__table_name__ = re.sub("_item$", "", self.name_underline)
        return self.__table_name__

    @table_name.setter
    def table_name(self, name):
        self.__table_name__ = name
        self.__name__ = tools.key2hump(name) + "Item"

    @property
    def name_underline(self):
        if not self.__name_underline__:
            self.__name_underline__ = tools.key2underline(self.item_name)

        return self.__name_underline__

    @name_underline.setter
    def name_underline(self, name):
        self.__name_underline__ = name

    @property
    def unique_key(self):
        return self.__unique_key__ or self.__class__.__unique_key__

    @unique_key.setter
    def unique_key(self, keys):
        if isinstance(keys, (tuple, list)):
            self.__unique_key__ = keys
        else:
            self.__unique_key__ = (keys,)

    @property
    def fingerprint(self):
        args = []
        for key, value in self.to_dict.items():
            if value:
                if (self.unique_key and key in self.unique_key) or not self.unique_key:
                    args.append(str(value))

        if args:
            args = sorted(args)
            return tools.get_md5(*args)
        else:
            return None

    def to_UpdateItem(self):
        update_item = UpdateItem(**self.__dict__)
        update_item.item_name = self.item_name
        return update_item


class UpdateItem(Item):
    __update_key__ = []

    def __init__(self, **kwargs):
        super(UpdateItem, self).__init__(**kwargs)

    @property
    def update_key(self):
        return self.__update_key__ or self.__class__.__update_key__

    @update_key.setter
    def update_key(self, keys):
        if isinstance(keys, (tuple, list)):
            self.__update_key__ = keys
        else:
            self.__update_key__ = (keys,)
